"""
Provides YAML parsing support.
"""
from __future__ import unicode_literals

from django.conf import settings
from django.utils import six
from rest_framework.exceptions import ParseError
from rest_framework.parsers import BaseParser

from .compat import yaml


class YAMLParser(BaseParser):
    """
    Parses YAML-serialized data.
    """

    media_type = 'application/yaml'

    def parse(self, stream, media_type=None, parser_context=None):
        """
        Parses the incoming bytestream as YAML and returns the resulting data.
        """
        assert yaml, 'YAMLParser requires pyyaml to be installed'

        parser_context = parser_context or {}
        encoding = parser_context.get('encoding', settings.DEFAULT_CHARSET)

        try:
            data = stream.read().decode(encoding)
            return yaml.safe_load(data)
        except (ValueError, yaml.parser.ParserError) as exc:
            raise ParseError('YAML parse error - %s' % six.text_type(exc))



"""
The `compat` module provides support for backwards compatibility with older
versions of django/python, and compatibility wrappers around optional packages.
"""
# flake8: noqa


try:
    import yaml
except ImportError:
    yaml = None

# OrderedDict only available in Python 2.7.
# This will always be the case in Django 1.7 and above, as these versions
# no longer support Python 2.6.
# For Django <= 1.6 and Python 2.6 fall back to OrderedDict.
try:
    from collections import OrderedDict
except:
    from django.utils.datastructures import SortedDict as OrderedDict

try:
    # Note: ReturnDict and ReturnList are private API from DRF 3
    from rest_framework.utils.serializer_helpers import ReturnDict, ReturnList
except ImportError:
    ReturnDict = None
    ReturnList = None



"""
Helper classes for parsers.
"""
from __future__ import unicode_literals
import decimal
import types

from django.utils import six

from .compat import yaml, OrderedDict, ReturnDict, ReturnList


class SafeDumper(yaml.SafeDumper):
    """
    Handles decimals as strings.
    Handles OrderedDicts as usual dicts, but preserves field order, rather
    than the usual behaviour of sorting the keys.

    Adapted from http://pyyaml.org/attachment/ticket/161/use_ordered_dict.py
    """
    def represent_decimal(self, data):
        return self.represent_scalar('tag:yaml.org,2002:str', six.text_type(data))

    def represent_mapping(self, tag, mapping, flow_style=None):
        value = []
        node = yaml.MappingNode(tag, value, flow_style=flow_style)
        if self.alias_key is not None:
            self.represented_objects[self.alias_key] = node
        best_style = True
        if hasattr(mapping, 'items'):
            mapping = list(mapping.items())
            if not isinstance(mapping, OrderedDict):
                mapping.sort()
        for item_key, item_value in mapping:
            node_key = self.represent_data(item_key)
            node_value = self.represent_data(item_value)
            if not (isinstance(node_key, yaml.ScalarNode) and not node_key.style):
                best_style = False
            if not (isinstance(node_value, yaml.ScalarNode) and not node_value.style):
                best_style = False
            value.append((node_key, node_value))
        if flow_style is None:
            if self.default_flow_style is not None:
                node.flow_style = self.default_flow_style
            else:
                node.flow_style = best_style
        return node

SafeDumper.add_representer(
    decimal.Decimal,
    SafeDumper.represent_decimal
)

SafeDumper.add_representer(
    OrderedDict,
    yaml.representer.SafeRepresenter.represent_dict
)

SafeDumper.add_representer(
    types.GeneratorType,
    yaml.representer.SafeRepresenter.represent_list
)

if ReturnDict:
    SafeDumper.add_representer(
        ReturnDict,
        yaml.representer.SafeRepresenter.represent_dict
    )

if ReturnList:
    SafeDumper.add_representer(
        ReturnList,
        yaml.representer.SafeRepresenter.represent_list
    )



"""
Provides YAML rendering support.
"""
from __future__ import unicode_literals

from rest_framework.renderers import BaseRenderer

from .compat import yaml
from .encoders import SafeDumper


class YAMLRenderer(BaseRenderer):
    """
    Renderer which serializes to YAML.
    """

    media_type = 'application/yaml'
    format = 'yaml'
    encoder = SafeDumper
    charset = 'utf-8'
    ensure_ascii = False

    def render(self, data, accepted_media_type=None, renderer_context=None):
        """
        Renders `data` into serialized YAML.
        """
        assert yaml, 'YAMLRenderer requires pyyaml to be installed'

        if data is None:
            return ''

        return yaml.dump(
            data,
            stream=None,
            encoding=self.charset,
            Dumper=self.encoder,
            allow_unicode=not self.ensure_ascii
        )
